package main

import (
	"fmt"
	"github.com/lesismal/arpc"
	"github.com/lesismal/arpc/extension/protocol/websocket"
	"log"
	"net"
	"net/http"
	"time"
)

var RPCServer *arpc.Server
var BroadcastClients = []*arpc.Client{}

func main()  {
	InitServer("8888")

	//another issue, comment InitServer, and uncomment code below, build as a client.
	//clientWithServer()
	//<- make(chan int)
}

func InitServer(port string)  {
	ln, _ := websocket.Listen("localhost:"+port, nil)

	http.HandleFunc("/", func(w http.ResponseWriter, r *http.Request) {
		fmt.Println("url: %v", r.URL.String())
		if r.URL.Path == "/" {
			http.ServeFile(w, r, "test_arpc.html")
		} else if r.URL.Path == "/arpc.js" {
			http.ServeFile(w, r, "arpc.js")
		} else {
			http.NotFound(w, r)
		}
	})

	http.HandleFunc("/ws", ln.(*websocket.Listener).Handler)
	go func() {
		err := http.ListenAndServe("localhost:"+port, nil)
		if err != nil {
			fmt.Println("ListenAndServe: ", err)
			panic(err)
		}
	}()

	server := arpc.NewServer()
	RPCServer = server
	/*
		Broadcast auth

		request  :
			AccessKey	string
		response :
			true or false
	*/
	server.Handler.Handle("/auth", func(context *arpc.Context) {
		var accessKey string
		if err := context.Bind(&accessKey); err != nil {
			context.Write(1)
			return
		}
		if len(accessKey) > 0 && accessKey == "abc"  {
			BroadcastClients = append(BroadcastClients, context.Client)
			context.Client.Handler.HandleDisconnected(func(clientWhichDisconnected *arpc.Client) {
				//remove from BroadcastClients
				idx := indexOfClient(clientWhichDisconnected,BroadcastClients)
				if idx >= 0 {
					if length := len(BroadcastClients); length != 1 {
						BroadcastClients[idx] = BroadcastClients[length - 1]
						BroadcastClients = BroadcastClients[:length - 1]
					}else {
						BroadcastClients = BroadcastClients[:0]
					}
				}
			});
			context.Write(0)
		}else {
			context.Write(1)
		}
	})

	//Push Data: broadcast current server info
	go broadcastServerInfo()

	//server.Run("localhost:"+port)
	server.Serve(ln)
}

func indexOfClient(element *arpc.Client, data []*arpc.Client) (int) {
	for k, v := range data {
		if element == v {
			return k
		}
	}
	return -1    //not found.
}

func broadcastServerInfo() {
	for true {
		if len(BroadcastClients) > 0 {
			log.Println(len(BroadcastClients), " clients listen Broad casting...")
			msg := RPCServer.NewMessage(arpc.CmdNotify,"/notify",
				"NewMessageNewMessageNewMessageNewMessageNewMessageNewMessageNewMessageRPCServerRPCServerRPCServerRPCServerRPCServerNewMessageNewMessageNewMessageNewMessageNewMessageNewMessageNewMes" +
				"NewMessageNewMessageNewMessageNewMessageNewMessageNewMessageNewMesNewMessageNewMessageNewMessageNewMessageNewMessageNewMessageNewMesNewMessageNewMessageNewMessageNewMessageNewMessageNewMessageNewMes" +
				"NewMessageNewMessageNewMessageNewMessageNewMessageNewMessageNewMesNewMessageNewMessageNewMessageNewMessageNewMessageNewMessageNewMes" +
				"NewMessageNewMessageNewMessageNewMessageNewMessageNewMessageNewMes,this is end of Data")
			for _, client := range BroadcastClients {
				client.PushMsg(msg, arpc.TimeZero)
			}
		}
		time.Sleep(time.Second * 1)
	}
}





//another issue

func clientWithServer() *arpc.Client  {
	client, err := arpc.NewClient(func() (net.Conn, error) {
		return net.DialTimeout("tcp", "localhost:8888", time.Second*3)
	})

	if err != nil {
		log.Println("Server Client Dial Build Error :",err)
		return nil
	}

	client.Handler.Handle("/notify",onBroadcastInfo)

	rsp := ""
	err = client.Call("/auth","abc",&rsp,time.Second * 5)
	if err != nil {
		log.Println("Server Client Build Error :",err)
		return nil
	}

	if rsp == "0" {
		return client
	}else {
		fmt.Println("Server Client result --- ",rsp )
		return nil
	}
}

func onBroadcastInfo(ctx *arpc.Context)  {

	var info string
	if err := ctx.Bind(&info); err != nil {
		fmt.Println("On Broad Error ", err)
	}
	fmt.Println("On Broad .. ", info)

}
